/***************************************************************************
 *   Copyright (C) 2009 by Mirko Lindner,,,   *
 *   vegyraupe@mira   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <gtkmm.h>
#include <zim/file.h>
#include <zim/search.h>
#include <iostream>
#include <string>
// #include <fstream>

#include "config.h"
#include "message_dialog.hh"
#include "vido.hh"
#include "main_window.hh"
#include "search_dialog.hh"

#include <cxxtools/loginit.h>

log_define("vido.main")

extern "C" {
   #include "gtkhtml/gtkhtml.h"
 }

std::string content;
std::string fileName;

main_window *window;
Gtk::Widget *html;
GtkWidget *html_wg;

// // // misc functions
// get zimFile, if not already opened, open file
const zim::File& get_file()
{
  static zim::File zimFile;
  if (!zimFile.good())
  {
    log_debug("file not initialized:" << fileName);
    zimFile = zim::File(fileName);
    log_debug("number of articles: " << zimFile.getCountArticles());

  }

  log_debug("returning file.");
  return zimFile;
}

// display message in gtk window
void show_message(std::string title, std::string txt)
{
    message_dialog(window, title, txt);
}

// fill gtkhtml widget with new content
//TODO prepend "top" anchor on html
void fill_gtkhtml(std::string& html){
    log_debug("fill gtkhtml called");
    gtk_html_flush(GTK_HTML(html_wg));
    gtk_html_load_from_string(GTK_HTML(html_wg), html.c_str(), -1);
}

//UNUSED
// // copy article html from while loop into global variable
void set_article(const std::string& txt)
{
    content = txt;
}

// // // externally called functions

// // display random article
void show_random()
{

    log_debug("random called.");
    zim::File m = get_file();
  
    unsigned int seed = static_cast<unsigned int>(time(0));
    zim::size_type idx = static_cast<zim::size_type>(static_cast<double>(m.getCountArticles()) * rand_r(&seed) / RAND_MAX);
    zim::Article article;
    
    article = m.getArticle(idx);
    
    //loop in case article is redirect
    do 
    {
        article = article.getRedirectArticle();
    }while(article.isRedirect());
    
    std::string res = article.getPage();

    content = article.getPage();
    log_debug("article size=" << content.size());
    log_debug("article title=" << article.getTitle());
    log_debug("article namespace=" << article.getNamespace());
    
    fill_gtkhtml(res);
//     gtk_html_flush(GTK_HTML(html_wg));
//     gtk_html_load_from_string(GTK_HTML(html_wg), res.c_str(), -1);
    //log_debug("html \n" << res.c_str());
}

// // display search dialog
void search_window(main_window *window_x)
{
    search_dialog(window_x, " ");
}

// // // meta functions

// // set displayed html to given url
// // FIXME: returns several articles and displays on one page ... why?
void getArticleFromUrl(const gchar *url)
{
    // TODO unescape url

// //     create ZIM file accessor
    zim::File m = get_file();

// //     convert url to string
    std::string term(url);

// //     replace '+' signs with spaces in url
    std::replace(term.begin(), term.end(), '+', ' ');
    
// //     find and declare namespace
    size_t found;
    size_t found2;
    char ns;

    found=term.find("/");
    if (found != std::string::npos)
    {
        if (found)
        {
            ns = term[0];
            term.erase(found + 1);
        }
        else
        {
            term.erase(1);
            ns = term[0];
            found2 = term.find("/");
            term.erase(found2 + 1);
        }
    }
    else
    {
        ns = 'A';
    }

// //     try to retrieve article
    try
    {
#if HAVE_ZIM_QUNICODE_H
        zim::Article article = m.getArticle(ns, zim::QUnicodeString(term));
#else
        zim::Article article = m.getArticle(ns, term);
#endif

        if (article.good())  // check if article is really found
        {
            content = article.getPage();

            fill_gtkhtml(content);
        }
        else
        {
            std::cerr << "article \"" << url << "\" not found" << std::endl;
            show_message("Error", "The article you requested (" + term + ")  was not found.");
        }
    }
    catch (const std::exception& e)
    {
        std::cerr << e.what() << std::endl;
    }
}


// // test functions
void getArticleFromTitle(const gchar *url)
{
    char ns;
    ns = 'A';
    zim::File z = get_file();
    
    zim::Search::Results result;
    zim::Search search(z);
    std::string term(url);
    std::string res;
    search.setSearchLimit(25);
    search.find(result, ns, term);
    if( result.size() == 0)
    {
        show_message("Error", "The article you requested (" + term + ")  was not found.");
    
    }else{ 
      
      if (result.size() == 1){
        log_debug("one article in result");
        
        zim::Article article = z.getArticle(result[0].getArticle().getIndex());
        
        //loop in case article is redirect
        do 
        {
            article = article.getRedirectArticle();
        }while(article.isRedirect());
        
        res = article.getPage(false, 10);
        
      }
      else
      {
        log_debug("more than one article in result");
        for (unsigned i = 0; i < result.size(); ++i)
        {
#if HAVE_ZIM_QUNICODE_H
          res += "<li><a href=" + result[i].getArticle().getLongUrl().toXML() + ">" + result[i].getArticle().getLongUrl().toXML() + "</a></li>";
#else
          res += "<li><a href='" + result[i].getArticle().getUrl() + "'>" + result[i].getArticle().getUrl() + "</a></li>";
#endif
        }
    
      }
      
      fill_gtkhtml(res);
    }
}

// // // window response functions

// // open requested link
// FIXME show error window when dead-link requested
bool on_link_clicked(GtkHTML *html, const gchar *url)
{
    std::string term(url);
    size_t found;
    found=term.find("#");
    if (found != std::string::npos){
    
      if (found == 0){
        term.erase(0, 1);
        bool success = gtk_html_jump_to_anchor(GTK_HTML(html_wg), const_cast<char*>(term.c_str()));
        log_debug("jump to " << term << "worked: " << success);
      }else{
        term = term.substr(0, found);   
        //TODO make "set_html" func accept anchor name
        getArticleFromUrl(const_cast<char*>(term.c_str()));
      }
      
    }else{
      getArticleFromUrl(url);
    }
    
    return true;
}


// // main function
int main(int argc, char **argv)
{
  try
  {
    log_init();

    if (argc == 2)
    {
      fileName = argv[1];

      log_debug("vido is here file: " << fileName);
      
      Gtk::Main kit(argc, argv);

      main_window window;
      window.set_title("Vido");
  //     window.set_border_width(0);
      window.set_default_size(220, 240);
  //     window.set_resizable(0);

      html_wg = gtk_html_new();
      
      html = Glib::wrap(html_wg);
      g_signal_connect( G_OBJECT( html_wg ), "link_clicked", G_CALLBACK( on_link_clicked ), NULL );
      gtk_html_set_caret_mode(GTK_HTML(html_wg),true);
     
      static Gtk::ScrolledWindow scrolled_window;
      scrolled_window.add(*html);
      scrolled_window.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC);
      
      window.add(scrolled_window);
      window.show_all();
      
      show_random();
      Gtk::Main::run(window);
    }
    else
    {
      std::cout << "usage: " << argv[0] << " [PATH TO ZIM FILE]" << std::endl;
    }

  }
  catch (const std::exception& e)
  {
    std::cerr << e.what() << std::endl;
    return -1;
  }

}
